'use strict';

/**
 * Compares two values.
 * @param a a value
 * @param b another value
 * @returns a<b: -1, a=b: 0, a>b: 1
 */
function cmp(a, b) {
    return a < b ? -1 : (a > b ? 1 : 0);
}

/**
 * Checks if array has a subsequence.
 * @param x an array
 * @param y subsequence?
 * @param fn compare function (a, b)
 */
function isSubsequence(x, y, fn = null) {
    if (y.length === 0)
        return true;
    var fn = fn || cmp;
    var j = 0, J = y.length;
    for (var u of x)
        if (fn(u, y[j]) === 0 && (++j) === J)
            return true;
    return false;
}

/**
 * Gives same value.
 * @param v a value
 * @returns v
 */
function id(v) {
    return v;
}

/**
 * Checks if array has a permutation.
 * @param x an array
 * @param y permutation?
 * @param fn map function (v, i, x)
 * @param ths this argument
 */
function isPermutationOn(x, y, fn = null, ths = null) {
    var fn = fn || id;
    var x1 = x.map(fn, ths).sort();
    var y1 = y.map(fn, ths).sort();
    return isSubsequence(x1, y1);
}

module.exports = isPermutationOn;
